@@ -38,6 +38,26 @@ static int check_prime( MPI prime, MPI val_2 );
 static int is_prime( MPI n, int steps, int *count );
 static void m_out_of_n( char *array, int m, int n );
 
+static void (*progress_cb) ( void *, int );
+static void *progress_cb_data;
+
+void
+register_primegen_progress ( void (*cb)( void *, int), void *cb_data )
+{
+    progress_cb = cb;
+    progress_cb_data = cb_data;
+}
+
+
+static void
+progress( int c )
+{
+    if ( progress_cb )
+	progress_cb ( progress_cb_data, c );
+    else
+	fputc( c, stderr );
+}
+
 
 /****************
  * Generate a prime number (stored in secure memory)
@@ -48,7 +68,7 @@ generate_secret_prime( unsigned  nbits )
     MPI prime;
 
     prime = gen_prime( nbits, 1, 2 );
-    fputc('\n', stderr);
+    progress('\n');
     return prime;
 }
 
@@ -58,7 +78,7 @@ generate_public_prime( unsigned  nbits )
     MPI prime;
 
     prime = gen_prime( nbits, 0, 2 );
-    fputc('\n', stderr);
+    progress('\n');
     return prime;
 }
 
@@ -110,8 +130,8 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	log_debug("gen prime: pbits=%u qbits=%u fbits=%u/%u n=%d\n",
 		    pbits, req_qbits, qbits, fbits, n  );
     prime = mpi_alloc( (pbits + BITS_PER_MPI_LIMB - 1) /  BITS_PER_MPI_LIMB );
-    q = gen_prime( qbits, 0, 1 );
-    q_factor = mode==1? gen_prime( req_qbits, 0, 1 ) : NULL;
+    q = gen_prime( qbits, 0, 0 );
+    q_factor = mode==1? gen_prime( req_qbits, 0, 0 ) : NULL;
 
     /* allocate an array to hold the factors + 2 for later usage */
     factors = m_alloc_clear( (n+2) * sizeof *factors );
@@ -138,7 +158,7 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	    perms = m_alloc_clear( m );
 	    for(i=0; i < n; i++ ) {
 		perms[i] = 1;
-		pool[i] = gen_prime( fbits, 0, 1 );
+		pool[i] = gen_prime( fbits, 0, 0 );
 		factors[i] = pool[i];
 	    }
 	}
@@ -147,12 +167,12 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	    for(i=j=0; i < m && j < n ; i++ )
 		if( perms[i] ) {
 		    if( !pool[i] )
-			pool[i] = gen_prime( fbits, 0, 1 );
+			pool[i] = gen_prime( fbits, 0, 0 );
 		    factors[j++] = pool[i];
 		}
 	    if( i == n ) {
 		m_free(perms); perms = NULL;
-		fputc('!', stderr);
+		progress('!');
 		goto next_try;	/* allocate new primes */
 	    }
 	}
@@ -169,8 +189,8 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	    if( ++count1 > 20 ) {
 		count1 = 0;
 		qbits++;
-		fputc('>', stderr);
-		q = gen_prime( qbits, 0, 1 );
+		progress('>');
+		q = gen_prime( qbits, 0, 0 );
 		goto next_try;
 	    }
 	}
@@ -180,8 +200,8 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	    if( ++count2 > 20 ) {
 		count2 = 0;
 		qbits--;
-		fputc('<', stderr);
-		q = gen_prime( qbits, 0, 1 );
+		progress('<');
+		q = gen_prime( qbits, 0, 0 );
 		goto next_try;
 	    }
 	}
@@ -190,7 +210,7 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
     } while( !(nprime == pbits && check_prime( prime, val_2 )) );
 
     if( DBG_CIPHER ) {
-	putc('\n', stderr);
+	progress('\n');
 	log_mpidump( "prime    : ", prime );
 	log_mpidump( "factor  q: ", q );
 	if( mode == 1 )
@@ -202,7 +222,7 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	    fprintf(stderr, ", q0=%u", mpi_get_nbits(q_factor) );
 	for(i=0; i < n; i++ )
 	    fprintf(stderr, ", p%d=%u", i, mpi_get_nbits(factors[i]) );
-	putc('\n', stderr);
+	progress('\n');
     }
 
     if( ret_factors ) { /* caller wants the factors */
@@ -237,7 +257,7 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 		mpi_print( stderr, g, 1 );
 	    }
 	    else
-		fputc('^', stderr);
+		progress('^');
 	    for(i=0; i < n+2; i++ ) {
 		/*fputc('~', stderr);*/
 		mpi_fdiv_q(tmp, pmin1, factors[i] );
@@ -247,7 +267,7 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 		    break;
 	    }
 	    if( DBG_CIPHER )
-		fputc('\n', stderr);
+		progress('\n');
 	} while( i < n+2 );
 	mpi_free(factors[n+1]);
 	mpi_free(tmp);
@@ -255,7 +275,7 @@ generate_elg_prime( int mode, unsigned pbits, unsigned qbits,
 	mpi_free(pmin1);
     }
     if( !DBG_CIPHER )
-	putc('\n', stderr);
+	progress('\n');
 
     m_free( factors );	/* (factors are shallow copies) */
     for(i=0; i < m; i++ )
@@ -335,7 +355,7 @@ gen_prime( unsigned  nbits, int secret, int randomlevel )
 		/* perform stronger tests */
 		if( is_prime(ptest, 5, &count2 ) ) {
 		    if( !mpi_test_bit( ptest, nbits-1 ) ) {
-			fputc('\n', stderr);
+			progress('\n');
 			log_debug("overflow in prime generation\n");
 			break; /* step loop, continue with a new prime */
 		    }
@@ -350,11 +370,11 @@ gen_prime( unsigned  nbits, int secret, int randomlevel )
 		}
 	    }
 	    if( ++dotcount == 10 ) {
-		fputc('.', stderr);
+		progress('.');
 		dotcount = 0;
 	    }
 	}
-	fputc(':', stderr); /* restart with a new random value */
+	progress(':'); /* restart with a new random value */
     }
 }
 
@@ -383,7 +403,7 @@ check_prime( MPI prime, MPI val_2 )
 	mpi_free( pminus1 );
 	if( mpi_cmp_ui( result, 1 ) ) { /* if composite */
 	    mpi_free( result );
-	    fputc('.', stderr);
+	    progress('.');
 	    return 0;
 	}
 	mpi_free( result );
@@ -392,7 +412,7 @@ check_prime( MPI prime, MPI val_2 )
     /* perform stronger tests */
     if( is_prime(prime, 5, &count ) )
 	return 1; /* is probably a prime */
-    fputc('.', stderr);
+    progress('.');
     return 0;
 }
 
@@ -452,7 +472,7 @@ is_prime( MPI n, int steps, int *count )
 	    if( mpi_cmp( y, nminus1 ) )
 		goto leave; /* not a prime */
 	}
-	fputc('+', stderr);
+	progress('+');
     }
     rc = 1; /* may be a prime */
 
