@@ -1,5 +1,5 @@
 /* armor.c - Armor flter
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -167,6 +167,7 @@ is_armored( const byte *buf )
     switch( pkttype ) {
       case PKT_MARKER:
       case PKT_SYMKEY_ENC:
+      case PKT_ONEPASS_SIG:
       case PKT_PUBLIC_KEY:
       case PKT_SECRET_KEY:
       case PKT_PUBKEY_ENC:
@@ -837,7 +838,8 @@ armor_filter( void *opaque, int control,
 		hashes &= 1|2|4|8;
 		if( !hashes ) {
 		    hashes |= 4;  /* default to MD 5 */
-		    afx->pgp2mode = 1;
+		    if( opt.pgp2_workarounds )
+			afx->pgp2mode = 1;
 		}
 		n=0;
 		do {
@@ -845,7 +847,7 @@ armor_filter( void *opaque, int control,
 		    buf[n++] = 0x90; /* old format, type 4, 1 length byte */
 		    buf[n++] = 13;   /* length */
 		    buf[n++] = 3;    /* version */
-		    buf[n++] = 0x01; /* sigclass 0x01 (canonical text mode)*/
+		    buf[n++] = afx->not_dash_escaped? 0:1; /* sigclass */
 		    if( hashes & 1 ) {
 			hashes &= ~1;
 			buf[n++] = DIGEST_ALGO_RMD160;
@@ -894,6 +896,8 @@ armor_filter( void *opaque, int control,
     }
     else if( control == IOBUFCTRL_FLUSH && !afx->cancel ) {
 	if( !afx->status ) { /* write the header line */
+	    const char *s;
+
 	    if( afx->what >= DIM(head_strings) )
 		log_bug("afx->what=%d", afx->what);
 	    iobuf_writestr(a, "-----");
@@ -903,26 +907,24 @@ armor_filter( void *opaque, int control,
 		iobuf_writestr(a, "Version: GnuPG v"  VERSION " ("
 					      PRINTABLE_OS_NAME ")" LF );
 
-	    if( opt.comment_string ) {
-		const char *s = opt.comment_string;
-		if( *s ) {
-		    iobuf_writestr(a, "Comment: " );
-		    for( ; *s; s++ ) {
-			if( *s == '\n' )
-			    iobuf_writestr(a, "\\n" );
-			else if( *s == '\r' )
-			    iobuf_writestr(a, "\\r" );
-			else if( *s == '\v' )
-			    iobuf_writestr(a, "\\v" );
-			else
-			    iobuf_put(a, *s );
-		    }
-		    iobuf_writestr(a, LF );
+	    /* write the comment string or a default one */
+	    s = opt.comment_string ? opt.comment_string
+				   : _("For info see http://www.gnupg.org");
+	    if( *s ) {
+		iobuf_writestr(a, "Comment: " );
+		for( ; *s; s++ ) {
+		    if( *s == '\n' )
+			iobuf_writestr(a, "\\n" );
+		    else if( *s == '\r' )
+			iobuf_writestr(a, "\\r" );
+		    else if( *s == '\v' )
+			iobuf_writestr(a, "\\v" );
+		    else
+			iobuf_put(a, *s );
 		}
+		iobuf_writestr(a, LF );
 	    }
-	    else
-		iobuf_writestr(a,
-		    "Comment: For info see http://www.gnupg.org" LF);
+
 	    if( afx->hdrlines )
 		iobuf_writestr(a, afx->hdrlines);
 	    iobuf_writestr(a, LF );
@@ -930,6 +932,7 @@ armor_filter( void *opaque, int control,
 	    afx->idx = 0;
 	    afx->idx2 = 0;
 	    afx->crc = CRCINIT;
+
 	}
 	crc = afx->crc;
 	idx = afx->idx;
@@ -1018,7 +1021,7 @@ armor_filter( void *opaque, int control,
 	    iobuf_put(a, c);
 	    c = bintoasc[radbuf[2]&077];
 	    iobuf_put(a, c);
-	    iobuf_put(a, '\n');
+	    iobuf_writestr(a, LF );
 	    /* and the the trailer */
 	    if( afx->what >= DIM(tail_strings) )
 		log_bug("afx->what=%d", afx->what);
