@@ -1,5 +1,5 @@
 /* encr-data.c -  process an encrypted data packet
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -32,9 +32,9 @@
 #include "i18n.h"
 
 
-static int decode_filter( void *opaque, int control, IOBUF a,
-					byte *buf, size_t *ret_len);
 static int mdc_decode_filter( void *opaque, int control, IOBUF a,
+					      byte *buf, size_t *ret_len);
+static int decode_filter( void *opaque, int control, IOBUF a,
 					byte *buf, size_t *ret_len);
 
 typedef struct {
@@ -76,9 +76,14 @@ decrypt_data( void *procctx, PKT_encrypted *ed, DEK *dek )
     if( ed->len && ed->len < (nprefix+2) )
 	BUG();
 
-    if( ed->mdc_method )
+    if( ed->mdc_method ) {
 	dfx.mdc_hash = md_open( ed->mdc_method, 0 );
-    dfx.cipher_hd = cipher_open( dek->algo, CIPHER_MODE_AUTO_CFB, 1 );
+	if ( DBG_HASHING )
+	    md_start_debug(dfx.mdc_hash, "checkmdc");
+    }
+    dfx.cipher_hd = cipher_open( dek->algo,
+				 ed->mdc_method? CIPHER_MODE_CFB
+					       : CIPHER_MODE_AUTO_CFB, 1 );
 /* log_hexdump( "thekey", dek->key, dek->keylen );*/
     rc = cipher_setkey( dfx.cipher_hd, dek->key, dek->keylen );
     if( rc == G10ERR_WEAK_KEY )
@@ -107,8 +112,6 @@ decrypt_data( void *procctx, PKT_encrypted *ed, DEK *dek )
 		temp[i] = c;
     }
     cipher_decrypt( dfx.cipher_hd, temp, temp, nprefix+2);
-    if( dfx.mdc_hash )
-	md_write( dfx.mdc_hash, temp, nprefix+2 );
     cipher_sync( dfx.cipher_hd );
     p = temp;
 /* log_hexdump( "prefix", temp, nprefix+2 ); */
@@ -116,29 +119,39 @@ decrypt_data( void *procctx, PKT_encrypted *ed, DEK *dek )
 	rc = G10ERR_BAD_KEY;
 	goto leave;
     }
+
+    if( dfx.mdc_hash )
+	md_write( dfx.mdc_hash, temp, nprefix+2 );
+
     if( ed->mdc_method )
 	iobuf_push_filter( ed->buf, mdc_decode_filter, &dfx );
     else
 	iobuf_push_filter( ed->buf, decode_filter, &dfx );
-    proc_packets( procctx, ed->buf);
+
+    proc_packets( procctx, ed->buf );
     ed->buf = NULL;
     if( ed->mdc_method && dfx.eof_seen == 2 )
 	rc = G10ERR_INVALID_PACKET;
     else if( ed->mdc_method ) { /* check the mdc */
 	int datalen = md_digest_length( ed->mdc_method );
+
+	cipher_decrypt( dfx.cipher_hd, dfx.defer, dfx.defer, 20);
 	md_final( dfx.mdc_hash );
 	if( datalen != 20
 	    || memcmp(md_read( dfx.mdc_hash, 0 ), dfx.defer, datalen) )
 	    rc = G10ERR_BAD_SIGN;
-	log_hexdump("MDC calculated:", md_read( dfx.mdc_hash, 0), datalen);
-	log_hexdump("MDC message   :", dfx.defer, 20);
+	/*log_hexdump("MDC calculated:", md_read( dfx.mdc_hash, 0), datalen);*/
+	/*log_hexdump("MDC message   :", dfx.defer, 20);*/
     }
+
   leave:
     cipher_close(dfx.cipher_hd);
     md_close( dfx.mdc_hash );
     return rc;
 }
 
+
+
 /* I think we should merge this with cipher_filter */
 static int
 mdc_decode_filter( void *opaque, int control, IOBUF a,
@@ -165,11 +178,14 @@ mdc_decode_filter( void *opaque, int control, IOBUF a,
 	}
 	if( n == 40 ) {
 	    /* we have enough stuff - flush the deferred stuff */
-	    /* (we have asserted that the buffer is large enough */
-	    if( !dfx->defer_filled ) /* the first time */
+	    /* (we have asserted that the buffer is large enough) */
+	    if( !dfx->defer_filled ) { /* the first time */
 		memcpy(buf, buf+20, 20 );
-	    else
+		n = 20;
+	    }
+	    else {
 		memcpy(buf, dfx->defer, 20 );
+	    }
 	    /* now fill up */
 	    for(; n < size; n++ ) {
 		if( (c = iobuf_get(a)) == -1 )
@@ -183,7 +199,7 @@ mdc_decode_filter( void *opaque, int control, IOBUF a,
 	    dfx->defer_filled = 1;
 	}
 	else if( !dfx->defer_filled ) { /* eof seen buf empty defer */
-	    /* this is very bad because there is an incomplete hash */
+	    /* this is bad because there is an incomplete hash */
 	    n -= 20;
 	    memcpy(buf, buf+20, n );
 	    dfx->eof_seen = 2; /* eof with incomplete hash */
@@ -217,16 +233,11 @@ decode_filter( void *opaque, int control, IOBUF a, byte *buf, size_t *ret_len)
     decode_filter_ctx_t *fc = opaque;
     size_t n, size = *ret_len;
     int rc = 0;
-    int c;
 
     if( control == IOBUFCTRL_UNDERFLOW ) {
 	assert(a);
-	for(n=0; n < size; n++ ) {
-	    if( (c = iobuf_get(a)) == -1 )
-		break;
-	    buf[n] = c;
-	}
-
+	n = iobuf_read( a, buf, size );
+	if( n == -1 ) n = 0;
 	if( n )
 	    cipher_decrypt( fc->cipher_hd, buf, buf, n);
 	else
