@@ -135,10 +135,11 @@ keygen_add_std_prefs( PKT_signature *sig, void *opaque )
 
     keygen_add_key_expire( sig, opaque );
 
-    buf[0] = CIPHER_ALGO_TWOFISH;
-    buf[1] = CIPHER_ALGO_BLOWFISH;
+    buf[0] = CIPHER_ALGO_RIJNDAEL;
+    buf[1] = CIPHER_ALGO_TWOFISH;
     buf[2] = CIPHER_ALGO_CAST5;
-    build_sig_subpkt( sig, SIGSUBPKT_PREF_SYM, buf, 3 );
+    buf[3] = CIPHER_ALGO_BLOWFISH;
+    build_sig_subpkt( sig, SIGSUBPKT_PREF_SYM, buf, 4 );
 
     buf[0] = DIGEST_ALGO_RMD160;
     buf[1] = DIGEST_ALGO_SHA1;
@@ -415,6 +416,8 @@ gen_dsa(unsigned int nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 }
 
 #if 0
+/* we can't enable generation right now, becuase we first need to implement
+ * the keyflags */
 static int
 gen_rsa(int algo, unsigned nbits, KBNODE pub_root, KBNODE sec_root, DEK *dek,
 	STRING2KEY *s2k, PKT_secret_key **ret_sk, u32 expireval )
@@ -793,7 +796,11 @@ ask_user_id( int mode )
 		aname = cpr_get("keygen.name",_("Real name: "));
 		trim_spaces(aname);
 		cpr_kill_prompt();
-		if( strpbrk( aname, "<([])>" ) )
+
+		if( opt.allow_freeform_uid )
+		    break;
+
+		if( strpbrk( aname, "<>" ) )
 		    tty_printf(_("Invalid character in name\n"));
 		else if( isdigit(*aname) )
 		    tty_printf(_("Name may not start with a digit\n"));
