@@ -84,6 +84,7 @@ struct {
     const char *set_policy_url;
     int use_embedded_filename;
     int allow_non_selfsigned_uid;
+    int allow_freeform_uid;
     int no_literal;
     ulong set_filesize;
     int honor_http_proxy;
@@ -91,6 +92,10 @@ struct {
     int ignore_time_conflict;
     int command_fd;
     int auto_key_retrieve;
+    const char *override_session_key;
+    int show_session_key;
+    int merge_only;
+    int try_all_secrets;
 } opt;
 
 
